
getApproxClusteredLinvLite <- function( covparms, covfun, locs, NNlist ){
    
    # 
    
    n <- nrow(locs)
    ngroups <- length(NNlist)
    Linv <- spam(0,n,n)
    
    print(paste("num groups:",ngroups))
    entrycount <- 0
    for( g in 1:ngroups){
        
        indarray <- NNlist[[g]]
        rowinds <- sort(indarray[,1])
        allinds <- sort(unique(c(indarray)), na.last = NA)
        likinds <- which( allinds %in% rowinds )
        entrycount <- entrycount + sum(likinds)
    }
    print("Done Counting")
    
    Lentries <- rep(NA,entrycount)
    rowpointers <- rep(NA,entrycount)
    colindices <- rep(NA,entrycount)
    
    entries_lastind <- 0
    
    for( g in 1:ngroups){
        
        indarray <- NNlist[[g]]
        rowinds <- sort(indarray[,1])
        allinds <- sort(unique(c(indarray)), na.last = NA)
        likinds <- which( allinds %in% rowinds )
        
        n0      <- length(allinds)
        locs0   <- locs[allinds,,drop=FALSE]
        #distmat <- rdist(locs0,locs0)
        covmat  <- covfun( locs0, covparms, returnD1=TRUE )
        cholmat <- tryCatch(t(chol(covmat)) , error = function(a) numeric(0) )
        attr(cholmat,"dmats") <- NULL
        
        for(k in 1:length(rowinds)){
            
            nk <- likinds[k]
            inds12 <- 1:nk
            
            subcovmat <- covmat[inds12,inds12]
            onevec     <- rep(0,nk)
            onevec[nk] <- 1
            M <- backsolve( t(cholmat[inds12,inds12]) , onevec )
            
            entry_inds <- entries_lastind + (1:nk)
            rowpointers[entry_inds] <- as.integer(rep(rowinds[k],nk))
            colindices[entry_inds]  <- as.integer(allinds[1:nk])
            Lentries[entry_inds] <- M
            
            entries_lastind <- entries_lastind + nk
        }
        
        if(g %% 400 == 0) print(g)
        
    }    
    Linv <- spam(list(i=rowpointers,j=colindices,Lentries),nrow=n,ncol=n)
    return(Linv)
}




maternSphereTime <- function(locstime,covparms,returnD1=FALSE){
    
    # isotropic matern in space, matern in time
    # locs <- (lon,lat,time)
    # nonseparable
    # covparms = (variance,spatial range,temporal range,smoothness,Sig2Noise)
    
    locs <- locstime[,1:2,drop=FALSE]
    time <- locstime[,3,drop=FALSE]
    chordaldist <- 2*sin(rdist.earth(locs,R=1)/2)
    timedist <- rdist(time)
    distmat <- sqrt( chordaldist^2/covparms[2]^2 + timedist^2/covparms[3]^2 )
    covmat <- maternFun(distmat,c(covparms[1],1,covparms[4],covparms[5]))
}




maternFun <- function(distmat, covparms, returnD1 = FALSE, returnD2 = FALSE, eee = 1e-8 ){
    
    # covariance parameters are...
    # (variance (including nugget), range, smoothness, signal to noise ratio)
    # range of values are...
    # (0,Inf), (0,Inf) , (0,Inf), (0,1) 
    
    n1 <- dim(distmat)[1]
    n2 <- dim(distmat)[2]
    covmat      <-  matrix(0,n1,n2)
    scaledist   <-  distmat/covparms[2]
    # to avoid bad spots for smoothness parameter
    covparms[3] <- max(covparms[3],1e-12)
    normcon     <-  2^(covparms[3]-1)*gamma(covparms[3])
    #sprint(covparms)
    
    if( covparms[3] == 1/2 ){ maternpart <- exp(-scaledist) }
    else if( covparms[3] == 3/2 ){ maternpart <- (1 + scaledist)*exp(-scaledist) }
    else if( covparms[3] == 1   ){ maternpart <- MaternFun(distmat,c(1,covparms[2:3])) }
    else {
        besspart <- besselK(scaledist,covparms[3])
        maternpart  <-  1/normcon*(scaledist)^covparms[3]*besspart
    }
    covmat[distmat != 0] <- covparms[1]*covparms[4]*maternpart[distmat!=0]
    covmat[distmat == 0] <- covparms[1]
    
    if(returnD1){
        
        if(!exists("besspart")) besspart <- besselK(scaledist,covparms[3])
        
        d1 <- covmat/covparms[1]
        
        d2 <- matrix(0,n1,n2)
        bessm1 <- besselK(scaledist,covparms[3]-1)
        bessp1 <- besselK(scaledist,covparms[3]+1)
        d2 <- -1/normcon*covparms[1]*covparms[4]*
            ( covparms[3]*scaledist^(covparms[3]-1)*besspart -
                  scaledist^(covparms[3])*(bessm1 + bessp1)/2 )*
            scaledist/covparms[2]
        d2[distmat==0] <- 0
        
        dcovparms <- covparms
        dcovparms[3] <- covparms[3]+eee
        d3 <- 1/eee*(maternFun(distmat,dcovparms)
                     -maternFun(distmat,covparms))
        
        d4 <- covmat/covparms[4]
        d4[distmat==0] = 0
        
        attr(covmat,"dmats") <- list( d1=d1, d2=d2, d3=d3, d4=d4 ) 
    }
    covmat
}
