
# install.packages("/Users/guinness/Dropbox/research/aldodevel_0.1.0.tar.gz", repos = NULL, type = "source")

library("aldodevel")
source("R/analyze_data_funs.R")


# which parts of the code to run
estimate_parms <- FALSE

# random seed
set.seed(031985)

# analyze the jason3 wind speed data
require(fields)
#require(spam)
#require(matrixStats)

# source .R source files
sapply(list.files(pattern="[.]R$", path="R/", full.names=TRUE), source);

# read in the original data
jason <- read.csv("jason3Aug04toAug09.txt",header=FALSE)
names(jason) <- c("lat0","lon0","time0","rainflag","iceflag","windspeed0")
attach(jason)
n0 <- length(lat0)


# flag any points within 30 seconds (buff) of rain or ice
buff <- 30
danwindow <- rep(0,n0)
danwindow[c(1:buff,(n0-buff+2):n0)] <- 1/(2*buff-1)  # daniell smoother

# runs flags through a moving averaged of width 60
# any value > 0 constitutes a flag
rainflag_sm <- Re(1/n0*fft( fft(rainflag)*fft(danwindow), inverse=TRUE ))
iceflag_sm <- Re(1/n0*fft( fft(iceflag)*fft(danwindow), inverse=TRUE ))


# inds1 holds indices to keep
inds1 <- abs(rainflag_sm) < 1e-9 & 
         abs(iceflag_sm) < 1e-9 & 
         !is.nan(windspeed0)
time1 <- time0[inds1]
lat1 <- lat0[inds1]
lon1 <- lon0[inds1]
windspeed1 <- windspeed0[inds1]
n1 <- length(lon1)


# compute R observation averages
R <- 10
j <- R
k <- 1
lon2 <- c()
lat2 <- c()
windspeed2 <- c()
time2 <- c()
while( j < n1-R ){
    
    # current R index span
    inds <- (k-1)*R + 1:R
    
    # index of last one (to tell when to stop)
    j <- inds[R]
    temptime <- time1[inds]
    
    # if the current R index time span is not larger than R+1,
    # then append R index averages to the var2 variables
    if( temptime[R] - temptime[1] < R+1 ){
        lon2 <- c(lon2,mean(lon1[inds]))
        lat2 <- c(lat2,mean(lat1[inds]))
        windspeed2 <- c(windspeed2,mean(windspeed1[inds]))
        time2 <- c(time2,mean(time1[inds]))
    }
    if( k %% 500 == 0 ) print(j)
    k <- k+1
}

# data are R second averages minus the mean
windspeed_zero <- windspeed2 - mean(windspeed2)
n <- length(windspeed2)

# save the dataset
dframe <- data.frame( windspeed = windspeed2, lon = lon2, lat = lat2, time = time2 )
#save(dframe,file="jason3_windspeed_avg.RData")
#write.table(dframe,file="jason3_windspeed_avg.txt")

# define the locations and the times
locs <- cbind(lon2,lat2)
locstime <- cbind(locs,time2)

# define the orderings

# random ordering
ord1 <- sample(n)

# time ordering
ord2 <- 1:n

# maxmin time ordering
nrecur <- 14
ord3 <- c()
for(j in 1:nrecur){
    ord3 <- c(ord3,round(n*seq( 2^(-j), 1 - 2^(-j), by = 2^(-j+1))))
}
ii <- 1:n
ii <- ii[-ord3]
ord3 <- c(ord3,sample(ii))

# maxmin distance ordering
lonrad <- dframe$lon*2*pi/360
latrad <- (dframe$lat+90)*2*pi/360
x <- sin(latrad)*cos(lonrad)
y <- sin(latrad)*sin(lonrad)
z <- cos(latrad)
xyz <- cbind(x,y,z)
ord4 <- spacefill_kdtree(xyz)

# 4D euclidean coordinates
xyztime <- cbind(xyz,time2)


t1 <- proc.time()
if( estimate_parms ){
#load("results0831.RData")
#saved_saved_pars <- saved_pars
#saved_saved_pars[,5,] <- 1-1e-9

allords <- cbind(ord1,ord2,ord3,ord4)
mvec <- seq(10,100,by=10)
prev_pars <- c(10,0.1,7000,1/2,0.001)
saved_pars <- array(NA,c(length(mvec),length(prev_pars),ncol(allords)))
saved_liks <- array(NA,c(length(mvec),ncol(allords)))
for(ordind in 1:4){
    ord <- allords[,ordind]
    for(m in 1:length(mvec)){
        
        # get ordering from spatial locations
        NNarray <- findOrderedNN_kdtree(xyz[ord,],mvec[m])
        NNlist <- groupNN(NNarray)
        X <- as.matrix( rep(1,length(windspeed2)) )
        Xord <- X[ord,,drop=FALSE] # don't actually need this since all ones
        xyztimeord <- xyztime[ord,]
        windspeedord <- windspeed_zero[ord]
        
        funtomax <- function( logparms ){
            
            # logparms are logarithm of parms
            # parms[1] = variance
            # parms[2] = spatial range
            # parms[3] = temporal range
            # parms[4] = smoothness
            # parms[5] = nugget
            parms <- exp(logparms)
            scaledcoords <- xyztimeord
            scaledcoords[,1:3] <- xyztimeord[,1:3]/parms[2]
            scaledcoords[,4] <- xyztimeord[,4]/parms[3]
            
            maternparms <- c( parms[1], 1, parms[4:5] )
            loglik <- OrderedGroupCompLik(maternparms, windspeedord, scaledcoords, NNlist )
            return(-loglik)
        }
        
        print(mvec[m])
        print(prev_pars)
        system.time( result <- optim(log(prev_pars),funtomax,control=list(trace=1,maxit=1000)) )
        prev_pars <- exp(result$par)
        saved_pars[m,,ordind] <- exp(result$par)
        saved_liks[m,ordind] <- -result$value
    
    }
}
save(saved_pars,saved_liks,file="results112817.RData")
}

proc.time() - t1




# load previously saved results
load("results112817.RData")





# create a grid of interpolation points
library(maptools)
data(wrld_simpl)

## Create a SpatialPoints object
lat_pred <- seq(-66,66,length.out=120)
lon_pred <- seq(0.1,359.9,length.out=240)
lon_pred_forflag <- lon_pred
lon_pred_forflag[lon_pred > 180] <- -360 + lon_pred[lon_pred > 180]
pnts_forflag <- expand.grid(lon_pred_forflag, lat_pred)  # Note that I reversed OP's ordering of lat/long
pts_forflag <- SpatialPoints(pnts_forflag, proj4string=CRS(proj4string(wrld_simpl)))

## Find which points fall over ocean
oceanflag <- is.na(over(pts_forflag, wrld_simpl)$FIPS)

# Check that it worked
locs_pred_all <- data.matrix(expand.grid(lon_pred,lat_pred))
locs_pred <- locs_pred_all[oceanflag,]
n_pred <- dim(locs_pred)[1]
plot(locs_pred[,1],locs_pred[,2],pch=16,cex=0.5)


# for the interpolations, use ordering 3 (AMMD) for data
# and random order for prediction locations
ord <- ord3
ord_pred <- sample(n_pred)
locs_pred_ord <- locs_pred[ord_pred,]
locs_both_ord <- rbind(locs[ord,],locs_pred_ord)
curtime   <- time2[n]                     # time for real-time mapping
halftime  <- mean(time2)                 # time for hindcasting
bothtimes <- c(halftime,curtime)

# get all of the ordered nearest neighbors  
NNarray_both <- findOrderedNN_kdtree(locs_both_ord,60)
NNlist_both <- groupNN(NNarray_both)

# covariance parameters to use
covparms <- saved_pars[10,,3]

# arrays to hold the predicted values and standard deviations
predarray <- array(NA,c(length(lon_pred),length(lat_pred),2))
sdarray   <- predarray

# conditional simulation settings
nsims <- 400
simmatrix <- array(0,c(n_pred,nsims,2))
y2hat <- matrix(0,n_pred,2)



for(tt in 1:2){

    # spacetime locations for prediction time tt    
    locstime_both_ord <- rbind(locstime[ord3,],cbind(locs_pred_ord,rep(bothtimes[tt],n_pred)))

    # compute L^{-1} (approximation to inverse cholesky)
    print(system.time({
        Linv <- getApproxClusteredLinvLite(covparms,maternSphereTime,
                        locstime_both_ord,NNlist_both)
    }))
    
    # solve to get prediction vector
    y10 <- c(y[ord3],rep(0,n_pred))
    Gy1 <- Linv %*% y10
    Gy1[1:n] <- 0
    sv <- forwardsolve( Linv, -Gy1 )
    y2hat[,tt] <- sv[(n+1):(n+n_pred)]

    # put the point predictions into an array suitable for image.plot()
    sv1 <- matrix(NA,length(lon_pred),length(lat_pred))
    sv1[which(oceanflag)[ord_pred]] <- y2hat[,tt] + mean(windspeed2)
    predarray[,,tt] <- sv1

    # simulate nsims times 
    for(j in 1:nsims){
        ysim <- forwardsolve(Linv, rnorm(n+n_pred))
        ysim0 <- c(ysim[1:n],rep(0,n_pred))
        Gysim0 <- Linv %*% ysim0
        Gysim0[1:n] <- 0
        ysimhat <- forwardsolve( Linv, -Gysim0 )[(n+1):(n+n_pred)]
        ycondsim <- y2hat[,tt] - ysim[(n+1):(n+n_pred)] + ysimhat
        simmatrix[,j,tt] <- ycondsim 
        
        if(j %% 100 == 0) print(j)
    }
}

# collect sds in to an array
sdvec <- matrix(0,n_pred,2)
for(tt in 1:2){ 
    sdvec[,tt] <- apply(simmatrix[,,tt]-y2hat[,tt],1,function(x) sqrt(mean( x^2 )))
    sv2 <- matrix(NA,length(lon_pred),length(lat_pred))
    sv2[which(oceanflag)[ord_pred]] <- sdvec[,tt]
    sdarray[,,tt] <- sv2
}


# make the plots
predarray[predarray>16] <- 16


#
#  conditional expectation and standard deviation plots
#

postscript("paper/figures/jason3_condexp_condsd.eps",width=6,height=3,horizontal = FALSE,
           onefile = FALSE, paper = "special")
layout(matrix(1:6,2,3,byrow=TRUE),widths = c(0.47,0.47,0.06),heights=c(1/2,1/2))
par(oma=c(0,1,1,1),mar=c(0.4,1,1,1))

# predictions speeds
image( predarray[,,1],zlim=c(0,16),col=tim.colors(64),axes=FALSE)
mtext("Predictions",2,line=0.5,cex=0.7)
mtext("Hindcast",3,line=0.5,cex=0.7)

image(predarray[,,2],zlim=c(0,16),axes=FALSE,col=tim.colors(64))
mtext("Real-Time Forecast",side=3,line=0.5,cex=0.7)

par(mar=c(0.4,0,1,2))
image(1,seq(0,16,length.out=65),t(as.matrix(1:64)),col=tim.colors(64),axes=FALSE)
box()
axis(4,at=seq(0,16,by=4),lwd=0,lwd.ticks=1,cex.axis=0.7)

# standard deviations
par(mar=c(0.4,1,1,1))
image( sdarray[,,1],zlim=c(0,3.5),col=tim.colors(64),axes=FALSE)
mtext("Standard Deviations",2,line=0.5,cex=0.7)

image(sdarray[,,2],zlim=c(0,3.5),axes=FALSE,col=tim.colors(64))

par(mar=c(0.4,0,1,2))
image(1,seq(0,3.5,length.out=65),t(as.matrix(1:64)),col=tim.colors(64),axes=FALSE)
box()
axis(4,at=seq(0,3,by=1),lwd=0,lwd.ticks=1,cex.axis=0.7)

dev.off()


#
# conditional simulation and conditional correlation plots
# 

postscript("paper/figures/jason3_condsim_condcorr.eps",width=6,height=3,horizontal = FALSE,
           onefile = FALSE, paper = "special")

layout(matrix(1:6,2,3,byrow=TRUE),widths = c(0.47,0.47,0.06),heights=c(1/2,1/2))
par(oma=c(0,1,1,1),mar=c(0.4,1,1,1))
tt <- 1

# conditional simulation plots
ind <- 1985
sv1 <- matrix(NA,length(lon_pred),length(lat_pred))
sv1[which(oceanflag)[ord_pred]] <- simmatrix[,ind,tt] + mean(windspeed2)
sv1[sv1<0] <- 0
image(sv1,axes=FALSE,zlim=c(0,22),col=tim.colors(64))
mtext("Conditional Simulations",2,line=0.5,cex=0.7)

ind <- 2016
sv1 <- matrix(NA,length(lon_pred),length(lat_pred))
sv1[which(oceanflag)[ord_pred]] <- simmatrix[,ind,tt] + mean(windspeed2)
sv1[sv1<0] <- 0
image(sv1,axes=FALSE,zlim=c(0,22),col=tim.colors(64))

par(mar=c(0.4,0,1,2))
image(1,seq(0,22,length.out=65),t(as.matrix(1:64)),col=tim.colors(64),axes=FALSE)
box()
axis(4,at=seq(0,22,by=5),lwd=0,lwd.ticks=1,cex.axis=0.7)

# conditional correlation plots
par(mar=c(0.4,1,1,1))
ind <- 685
print(ind)
corrvec <- 1/nsims*( (simmatrix[,,tt]- y2hat[,tt]) %*% (simmatrix[ind,,tt]-y2hat[ind,tt]))/(sdvec[,tt]*sdvec[ind,tt])
sv1 <- matrix(NA,length(lon_pred),length(lat_pred))
sv1[which(oceanflag)[ord_pred]] <- corrvec
image(sv1[161:240,1:40],axes=FALSE,zlim=c(-0.1,1),col=tim.colors(64))
mtext("Conditional Correlations",2,line=0.5,cex=0.7)

ind <- 19946
print(ind)
corrvec <- 1/nsims*( (simmatrix[,,tt]- y2hat[,tt]) %*% (simmatrix[ind,,tt]-y2hat[ind,tt]))/(sdvec[,tt]*sdvec[ind,tt])
sv1 <- matrix(NA,length(lon_pred),length(lat_pred))
sv1[which(oceanflag)[ord_pred]] <- corrvec
image(sv1[161:240,1:40],axes=FALSE,zlim=c(-0.1,1),col=tim.colors(64))

par(mar=c(0.4,0,1,2))
image(1,seq(-0.1,1,length.out=65),t(as.matrix(1:64)),col=tim.colors(64),axes=FALSE)
box()
axis(4,at=seq(0,1,by=0.25),lwd=0,lwd.ticks=1,cex.axis=0.7)

dev.off()








