

# plots for data section of reordering paper

load("jason3_windspeed_avg.RData")
#attach(dframe)
library(fields)

makeplot1 <- FALSE
makeplot2 <- FALSE
makeplot3 <- FALSE
makeplot4 <- FALSE
makeplot5 <- TRUE

if( makeplot1 ){
# 1x2 plot of times and wind speeds
colmap <- tim.colors(64)
col1 <- colmap[as.numeric(cut(time/3600,breaks=64))]
breaks2 <- seq(min(windspeed),max(windspeed),length.out=65)
col2 <- colmap[as.numeric(cut(windspeed,breaks=breaks2))]

postscript("paper/figures/jason3data.eps",width=6,height=2.5,horizontal = FALSE,
           onefile = FALSE, paper = "special")

par(oma=rep(0,4))
layout(matrix(c(1,2,3,4),2,2),widths=c(0.5,0.5),heights=c(0.75,0.25))

# wind speed plot
par(mar=c(0,1,1,1))
plot(lon,lat,col=col2,pch=16,cex=0.2,axes=FALSE,ann=FALSE)
box()
par(mar=c(2.5,1,0.5,1))
windrange <- range(windspeed)
windvec <- seq(0,windrange[2],length.out=64)
image(windvec,1,as.matrix(windvec),
      col=tim.colors(64),axes=FALSE)
box()
axis(1,at=seq(0,20,by=2),labels=rep("",11),lwd=0,lwd.ticks=1)
mtext(seq(0,20,by=2),1,at=seq(0,20,by=2),line=0.6,cex=0.7)
mtext("Wind Speed (m/s)",1,at=10,line=1.5,cex=0.7)

# time plot
par(mar=c(0,1,1,1))
plot(lon,lat,col=col1,pch=16,cex=0.2,axes=FALSE,ann=FALSE)
box()
par(mar=c(2.5,1,0.5,1))
timerange <- range(time)/3600
timevec <- seq(timerange[1],timerange[2],length.out=64)
image(timevec,1,as.matrix(timevec),
      col=tim.colors(64),axes=FALSE)
box()
axis(1,at=seq(0,144,by=24),labels=rep("",7),lwd=0,lwd.ticks=1)
mtext(seq(0,144,by=24),1,at=seq(0,144,by=24),line=0.6,cex=0.7)
mtext("Time (hours)",1,at=72,line=1.5,cex=0.7)



dev.off()
}


if(makeplot2){

# plots of sequential parameter estimates
load("results0830.RData")
sv1 <- saved_pars
sv2 <- saved_liks
load("results0831.RData")
sv3 <- saved_pars
sv4 <- saved_liks
load("results0901.RData")
saved_pars <- abind::abind( sv1, sv3, saved_pars, along = 1 )
saved_pars[,3,] <- saved_pars[,3,]/3600
saved_pars[,2,] <- saved_pars[,2,]*6371
saved_liks <- abind::abind( sv2, sv4, saved_liks, along = 1 )
mvec <- seq(10,100,by=10)
yheights <- 0.2*saved_pars[10,1:4,3]
yloval <- c(10.4,637,17,0.68)
ylimits <- cbind(yloval,yloval+yheights)
ylabvec <- c("variance","spatial range (km)","temporal range (hours)","smoothness")

postscript("paper/figures/jason_parm_convergence.eps",width=6,height=2.5,horizontal = FALSE,
           onefile = FALSE, paper = "special")
par(mfrow=c(1,4),oma=c(0,0,0,0),mar=c(5,2,2,0.5))
for(j in 1:4){
    plot(mvec,saved_pars[,j,1],ylim=ylimits[j,],type="o",col="magenta",
         xlab="# of neighbors",ylab="",pch=16)
    lines(mvec,saved_pars[,j,2],type="o",col="black",pch=16)
    lines(mvec,saved_pars[,j,3],type="o",col="blue",pch=16)
    mtext(ylabvec[j],3,line=1,cex=0.7)
    if(j==1){
        points(rep(50,3),c(12.4,12.2,12.0),pch=16,col=c("black","magenta","blue"))
        text(50,12.4,"time order",pos=4)
        text(50,12.2,"random",pos=4)
        text(50,12.0,"AMMD",pos=4)
    }
}
dev.off()

}



if(makeplot3){

# make a plot of the first 1000 points
nrecur <- 14
n <- length(lon)
ord3 <- c()
for(j in 1:nrecur){
    ord3 <- c(ord3,round(n*seq( 2^(-j), 1 - 2^(-j), by = 2^(-j+1))))
}
ii <- 1:n
ii <- ii[-ord3]
ord3 <- c(ord3,sample(ii))


postscript("paper/figures/jason_first_1000.eps",width=6,height=3,horizontal = FALSE,
           onefile = FALSE, paper = "special")
par(oma=c(0,0,0,0),mar=c(0,0,0,0))
plot( lon, lat, col = "gray", pch = 16, cex = 0.2, xlim=c(0,360),ylim=c(-70,70), 
      axes= FALSE, ann = FALSE)
points(lon[ord3[1:1000]], lat[ord3[1:1000]], pch =16, col = "black", cex = 0.5 )
dev.off()

}






if( makeplot4 ){
    # plot of wind speeds
    colmap <- tim.colors(64)
    col1 <- colmap[as.numeric(cut(time/3600,breaks=64))]
    breaks2 <- seq(min(windspeed),max(windspeed),length.out=65)
    col2 <- colmap[as.numeric(cut(windspeed,breaks=breaks2))]
    
    postscript("paper/figures/jason3data_notimes.eps",width=6,height=3,horizontal = FALSE,
               onefile = FALSE, paper = "special")

    # wind speed plot
    par(mar=c(1,1,1,1/4),oma=c(0,0,0,3))
    layout(matrix(c(1,2),1,2),widths=c(0.92,0.08),heights=c(1))
    plot(lon,lat,col=col2,pch=16,cex=0.2,axes=FALSE,ann=FALSE)
    box()
    #par(mar=c(2.5,1,0.5,1))
    windrange <- range(windspeed)
    windvec <- seq(0,windrange[2],length.out=64)
    image(1,windvec,t(as.matrix(windvec)),
          col=tim.colors(64),axes=FALSE)
    box()
    axis(4,at=seq(0,20,by=4),labels=rep("",6),lwd=0,lwd.ticks=1)
    mtext(seq(0,20,by=4),4,at=seq(0,20,by=4),line=0.6,cex=0.7)
    mtext("Wind Speed (m/s)",4,at=10,line=1.5,cex=0.7)
    
    dev.off()
}






if(makeplot5){
    
    # plots of sequential parameter estimates
    load("results112817.RData")
    saved_pars[,3,] <- saved_pars[,3,]/3600
    saved_pars[,2,] <- saved_pars[,2,]*6371
    mvec <- seq(10,100,by=10)
    yheights <- 0.2*saved_pars[10,1:4,3]
    yloval <- c(10.4,637,17,0.68)
    ylimits <- cbind(yloval,yloval+yheights)
    ylabvec <- c("variance","spatial range (km)","temporal range (hours)","smoothness")
    
    pdf("paper/figures/jason_parm_convergence.pdf",width=6,height=2.5)
    par(mfrow=c(1,4),oma=c(0,0,0,0),mar=c(5,2,2,0.5))
    for(j in 1:4){
        plot(mvec,saved_pars[,j,1],ylim=ylimits[j,],type="o",col="purple",
             xlab="# of neighbors",ylab="",pch=16)
        lines(mvec,saved_pars[,j,2],type="o",col="black",pch=16)
        lines(mvec,saved_pars[,j,3],type="o",col="blue",pch=16)
        lines(mvec,saved_pars[,j,4],type="o",col="red",pch=16)
        mtext(ylabvec[j],3,line=1,cex=0.7)
        if(j==1){
            points(rep(50,4),c(12.4,12.2,12.0,11.8),pch=16,col=c("black","purple","blue","red"))
            text(50,12.4,"time order",pos=4,cex=0.9)
            text(50,12.2,"random",pos=4,cex=0.9)
            text(50,12.0,"MMD Time",pos=4,cex=0.9)
            text(50,11.8,"MMD Space",pos=4,cex=0.9)
        }
    }
    dev.off()
    
}







