

# 3D and 4D kl divergence

require(fields)
require(spam)
require(matrixStats)
#library(Rcpp)
library(FNN)
#sourceCpp("src/MaternFun.cpp")



sapply(list.files(pattern="[.]R$", path="R/", full.names=TRUE), source);


set.seed(031985)

# create a grid whose size is close to ntarget
ntarget <- 6400
for( dim in 3:4 ){
nn <- round(ntarget^(1/dim))
nvec <- rep(nn,dim)
n <- prod(nvec)
#locs <- simulateGrid(nvec,errorsize=0.00)
if(dim==2){
    locs <- as.matrix( expand.grid( (1:nvec[1])/nvec[1], (1:nvec[2])/nvec[2] ) )
}
if(dim==3){
    locs <- as.matrix( 
        expand.grid( (1:nvec[1])/nvec[1], (1:nvec[2])/nvec[2], (1:nvec[3])/nvec[3] ) 
    )
} else if(dim==4){
    locs <- as.matrix( 
        expand.grid( (1:nvec[1])/nvec[1], (1:nvec[2])/nvec[2], 
                     (1:nvec[3])/nvec[3], (1:nvec[4])/nvec[4] ) 
    )
}
covfun <- maternIsotropic

rngvec <- c(10,20,40)
smvec <- c(5)


mvec <- seq(30,240,by=30)
mm <- length(mvec)

# generate some orderings
ordermat <- matrix(0,n,5)
ordermat[,1] <- spacefill_kdtree(locs)     # need to rewrite nD ordering code 
ordermat[,2] <- sample(n)
ordermat[,3] <- order(locs[,1])
ordermat[,4] <- order(locs[,1])
ordermat[,5] <- orderMiddleOut(locs)

# compute a distance matrix
distmat <- rdist(locs,locs)

# get blocks for block composite likelihood
#blocklist1 <- getBlockList(locs,2,2)
#blocklist2 <- getBlockList(locs,3,3)


#spdeoffset <- rbind(c(0.2,0.2),c(0.2,0.4),c(0.2,0.2),c(0.2,0.4))
#spdemaxedge <- rbind(c(0.05,0.1),c(0.05,0.1),c(0.1,0.2),c(0.1,0.2))

kldiv  <- array(0,c(mm, ncol(ordermat) , 5 , length(rngvec) , length(smvec) ) )
timing <- array(0,c(mm, ncol(ordermat) , 5 , length(rngvec) , length(smvec) ) )



for(r in 1:length(rngvec)){
    for(s in 1:length(smvec)){

        covparms <- c(variance = 1, range = rngvec[r]/100, smoothness = smvec[s]/10, S2N = 1)
        cat(paste0("\n Range = ",covparms[2],", smoothness = ",covparms[3]),"\n\n")
        
        # -1/2 log determinant plus constant
        zvec <- rep(0,n)
        logdet0 <- mvnMargLik(covparms,covfun,zvec,locs)
        
        for( k in 1:mm){
            m <- mvec[k]
            print(m)
            for(j in 1:ncol(ordermat)){
                ord <- ordermat[,j]
                locsord <- locs[ord,]
                if( j == 4 ){ NNarray <- findOrderedNNdistant(locsord,m,0.75) 
                } else { NNarray <- findOrderedNN_kdtree(locsord,m) }
                
                NNlist <- clusterNN2(NNarray)
                
                timing[k,j,1,r,s] <- system.time( kldiv[k,j,1,r,s] <- logdet0 - orderedCompLik(covparms,covfun,zvec,locsord,NNarray) )[3]
                timing[k,j,2,r,s] <- system.time( kldiv[k,j,2,r,s] <- logdet0 - orderedClusterCompLik(covparms,covfun,zvec,locsord,NNlist) )[3]
                #if(j==1 && k==1) timing[k,j,3,r,s] <- system.time( kldiv[k,j,3,r,s] <- logdet0 - mvnIndepBlocks(covparms,covfun,zvec,locs,blocklist1) )[3]
                #if(j==1 && k==1) timing[k,j,4,r,s] <- system.time( kldiv[k,j,4,r,s] <- logdet0 - mvnIndepBlocks(covparms,covfun,zvec,locs,blocklist2) )[3]
            }
        }

        # tapering comparison
        if(smvec[s]==5){
            covmat <- maternFun(distmat,covparms)
            taper_range <- seq(0.16,0.24,by=0.04)
            taper_k = c(1,2)
            for( tk in 1:1 ){  # just use k=1
                for( tr in 1:3 ){
                    timing[tk,tr,5,r,s] <- system.time({
                        tapercov <- stationary.taper.cov(locs,theta=covparms[2],Taper.args = list(theta=taper_range[tr],k=taper_k[tk]))
                        choltaper <- chol.spam(tapercov)
                    })[3]
                    funtomax <- function(x){ taperfun(locs,exp(x[1]),exp(x[2]),taper_range[tr],taper_k[tk],covmat) }
                    result <- optim( log(covparms[1:2]), funtomax, control=list(trace=5) )
                    kldiv[tk,tr,5,r,s] <- result$value
                }
            }
        }
        
    }
}


fname <- paste0("results/numstudy_",dim,"D.RData")
save(file=fname,list=c("kldiv","timing"))

}

