

# compute the kl divergence
#install.packages("INLA", repos="https://inla.r-inla-download.org/R/stable", dep=TRUE)
#install.packages("aldodevel_0.1.0.tar.gz",repos = NULL, type = "source")

library(fields)
library(matrixStats)
library(INLA)
source("R/numstudy_funs.R")
library("aldodevel")



# set up a numerical study of relative efficiencies
# for different orderings, same # of neighbors

set.seed(031985)

# nn given as input from command line
nn <- 80
nvec <- c(nn,nn)
n <- prod(nvec)
#locs <- simulateGrid(nvec,errorsize=0.00)
locs <- as.matrix( expand.grid( (1:nn)/nn, (1:nn)/nn ) )
covfun <- maternIsotropic

rngvec <- c(10,20)
smvec <- c(5,10,15)


mvec <- seq(30,60,by=30)
mm <- length(mvec)
kldiv  <- array(0,c(mm,5,6,2,3))
timing <- array(0,c(mm,5,6,2,3))

# generate some orderings
ordermat <- matrix(0,n,5)
ordermat[,1] <- orderMaxMinLocal(locs) 
ordermat[,2] <- sample(n)
ordermat[,3] <- order(locs[,1])
ordermat[,4] <- order(locs[,1])
ordermat[,5] <- orderMiddleOut(locs)

# compute a distance matrix
distmat <- rdist(locs,locs)

# get blocks for block composite likelihood
blocklist1 <- getBlockList(locs,2,2)
blocklist2 <- getBlockList(locs,3,3)


spdeoffset <- rbind(c(0.2,0.2),c(0.2,0.4),c(0.2,0.2),c(0.2,0.4))
spdemaxedge <- rbind(c(0.05,0.1),c(0.05,0.1),c(0.1,0.2),c(0.1,0.2))

for(r in 1:length(rngvec)){
    for(s in 1:length(smvec)){
        covparms <- c(variance = 1, range = rngvec[r]/100, smoothness = smvec[s]/10, nugget = 0)
        cat(paste0("\n Range = ",covparms[2],", smoothness = ",covparms[3]),"\n\n")
                     
        # -1/2 log determinant plus constant
        zvec <- rep(0,n)
        logdet0 <- exactLik(covparms,covfun,zvec,locs)
        
        for( k in 1:mm){
            m <- mvec[k]
            print(m)
            for(j in 1:ncol(ordermat)){
                ord <- ordermat[,j]
                locsord <- locs[ord,]
                if( j == 4 ){ NNarray <- findOrderedNNdistant(locsord,m,0.75) 
                } else { NNarray <- findOrderedNN_kdtree(locsord,m) }
        
                NNlist <- groupNN(NNarray)
        
                timing[k,j,1,r,s] <- system.time( kldiv[k,j,1,r,s] <- logdet0 - OrderedCompLik_chol(covparms,"maternIsotropic",zvec,locsord,NNarray) )[3]
                timing[k,j,2,r,s] <- system.time( kldiv[k,j,2,r,s] <- logdet0 - OrderedGroupCompLik_chol(covparms,zvec,locsord,NNlist) )[3]
                if(j==1 && k==1) timing[k,j,3,r,s] <- system.time( kldiv[k,j,3,r,s] <- logdet0 - mvnIndepBlocks(covparms,covfun,zvec,locs,blocklist1) )[3]
                if(j==1 && k==1) timing[k,j,4,r,s] <- system.time( kldiv[k,j,4,r,s] <- logdet0 - mvnIndepBlocks(covparms,covfun,zvec,locs,blocklist2) )[3]
            }
        }
        
        # # tapering comparison
        if(smvec[s]==5){
            covmat <- maternFun(distmat,covparms)
            taper_range <- seq(0.08,0.16,by=0.04)
            taper_k = c(1,2)
            for( tk in 1:1 ){  # just use k=1
                for( tr in 1:3 ){
                    timing[tk,tr,6,r,s] <- system.time({
                        tapercov <- stationary.taper.cov(locs,theta=covparms[2],Taper.args = list(theta=taper_range[tr],k=taper_k[tk]))
                        choltaper <- chol.spam(tapercov)
                    })[3]
                    funtomax <- function(x){ taperfun(locs,exp(x[1]),exp(x[2]),taper_range[tr],taper_k[tk],covmat) }
                    result <- optim( log(covparms[1:2]), funtomax, control=list(trace=0) )
                    kldiv[tk,tr,6,r,s] <- result$value
                }
            }
        }

        if(smvec[s]==10){
            covmat <- maternIsotropic(locs,covparms)
            cholmat <- t(chol(covmat))
            for( ix in 1:dim(spdeoffset)[1]){
                funtomax <- function(x){ spdefun(locs,exp(x[1]),exp(x[2]),offset=spdeoffset[ix,],max.edge=spdemaxedge[ix,],getKL=TRUE,covmat=covmat,cholmat=cholmat) }
                result <- optim(log(c(0.5,0.05)),funtomax,control = list(trace=5))
                kldiv[1,ix,5,r,s] <- result$value
                timing[1,ix,5,r,s] <- system.time({
                    Q <- spdefun(locs,exp(result$par[1]),exp(result$par[2]),offset=spdeoffset[ix,],max.edge=spdemaxedge[ix,])
                    cQ <- Cholesky(Q)
                    z <- solve(cQ,rep(0,nrow(Q)))
                })[3]
            }
        }
    }
}


#fname <- paste("results/numstudy_n",nvec[1],"_KL_timing_taper_spde_cpp5.Rdata",sep="")
#save(file=fname,list=c("kldiv","timing"))


