

library(fields)
library(spam)
library(profvis)
library(matrixStats)
sapply(list.files(pattern="[.]R$", path="R/", full.names=TRUE), source);

# make plots of the ordering schemes
set.seed(3)

nvec <- c(20,20)
n <- prod(nvec)
locs <- simulateGrid(nvec,errorsize=0.01)

# generate some orderings
ordermat <- matrix(0,n,4)
ordermat[,1] <- sample(n)
ordermat[,2] <- orderMaxMinFast(locs,n) 
ordermat[,3] <- orderMiddleOut(locs) 
ordermat[,4] <- 1:n

# 8 plots (two order points)
postscript(file="paper/figures/orderplots8.eps",width=12,height=6,horizontal = FALSE, 
           onefile = FALSE, paper = "special")
par(mfrow=c(2,4),mar=c(1,1,2,1))
# plots of the orderings
k <- 50
m <- 30
for( j in 1:ncol(ordermat)){
    cx <- 1.5
    ord <- ordermat[,j]
    locsord <- locs[ord,]
    NNarray <- findOrderedNN(locsord,m)
    plot(locsord[,1],locsord[,2],pch=16,col="gray",cex=cx,axes=F,
         xlab="",ylab="",cex.main=2)
    points(locsord[1:k,1],locsord[1:k,2],pch=16,cex=cx)
    points(locsord[k,1],locsord[k,2],cex=2*cx)
    points(locsord[NNarray[k,2:(m+1)],1],locsord[NNarray[k,2:(m+1)],2],cex=1.5*cx,pch=0)
}

k <- 330
m <- 30
for( j in 1:ncol(ordermat)){
    cx <- 1.5
    ord <- ordermat[,j]
    locsord <- locs[ord,]
    NNarray <- findOrderedNN(locsord,m)
    plot(locsord[,1],locsord[,2],pch=16,col="gray",cex=cx,axes=F,
         xlab="",ylab="",cex.main=2)
    points(locsord[1:k,1],locsord[1:k,2],pch=16,cex=cx)
    points(locsord[k,1],locsord[k,2],cex=2*cx)
    points(locsord[NNarray[k,2:(m+1)],1],locsord[NNarray[k,2:(m+1)],2],cex=1.5*cx,pch=0)
}
dev.off()

