

# timing results for increasing number of observations
require(fields)
#require(spam)
#require(matrixStats)
#sapply(list.files(pattern="[.]R$", path="R/", full.names=TRUE), source);
set.seed(031985)
library("aldodevel")
source("R/numstudy_funs.R")

#nnvec <- round( sqrt( seq(4000,100000,by=4000) ) )
nnvec <- round( sqrt( exp(seq(log(4000),log(100000),length.out = 10) )) )

covfun <- maternIsotropic
covparms <- c(1,0.1,1/2,1)
mvec <- c(30,60)

timing <- array(0,c(length(nnvec),9,5))
for(j in 1:length(nnvec)){
    
    nvec <- c(nnvec[j],nnvec[j])
    locs <- as.matrix(expand.grid( (1:nvec[1])/nvec[1], (1:nvec[2])/nvec[2] ) )
    n <- prod(nvec)
    
    y <- rnorm(n)
    # time MMD ordering
    timing[j,1,] <- system.time( ord <- orderMaxMinLocal(locs) )
    yord <- y[ord]
    locsord <- locs[ord,]
    timing[j,2,] <- system.time( NN1 <- findOrderedNN_kdtree(locs,mvec[1]) )
    timing[j,3,] <- system.time( NN2 <- findOrderedNN_kdtree(locs,mvec[2]) )
    timing[j,4,] <- system.time( NNlist1 <- groupNN(NN1) )
    timing[j,5,] <- system.time( NNlist2 <- groupNN(NN2) )
    timing[j,6,] <- system.time( OrderedCompLik_chol(covparms,"maternIsotropic",yord,locsord,NN1) )
    timing[j,7,] <- system.time( OrderedCompLik_chol(covparms,"maternIsotropic",yord,locsord,NN2) )
    timing[j,8,] <- system.time( OrderedGroupCompLik_chol(covparms,yord,locsord,NNlist1) )
    timing[j,9,] <- system.time( OrderedGroupCompLik_chol(covparms,yord,locsord,NNlist2) )
    print(n)
}    
print(timing)



linex <- c(3.6)
liney <- seq(1.0,2.0,length.out=5)
textvec <- c("order","neighbors","grouping","ungrouped lik","grouped lik")
subinds <- 1:length(nnvec)
inds <- cbind(c(1,2,4,6,8),c(1,3,5,7,9))
colvec <- c("blue","red","cyan","black","magenta")


pdf("paper/figures/timing_increasing_n.pdf",width=6,height=3)
par(mfrow=c(1,2),oma=c(0,0,0,0),mar=c(4,4,2,0.5))
for(k in 1:2){
plot(0,type="n",xlim=c(3.6,5),ylim=c(-1.2,2.1),axes=FALSE,ann=FALSE)
for(j in log10(2^(-4:7))) abline(j,0,col="gray",cex=0.5)
for(j in 1:5) 
    lines(log10(nnvec[subinds]^2),log10(timing[subinds,inds[j,k],3]),
          type="o",col=colvec[j],pch=15)
axis(1,at=log10(1000*c(2^(2:7),100)),
     labels=c(c(2^(2:7),100)),lwd=0,lwd.ticks=1,cex=0.7,cex.axis=0.5)
axis(2,at=log10(c(1/16,1/4,1,4,16,64)),
     labels=c("1/16","1/4",1,4,16,64),lwd=0,lwd.ticks=1,cex=0.7,cex.axis=0.5)
box()
for(j in 1:5) points(linex,liney[j],col=colvec[j],pch=15)
for(j in 1:5) text(linex,liney[j],textvec[j],pos=4,cex=0.7)
mtext("# of observations (thousands)",1,line=2.5,cex=0.7)
mtext("time (seconds)",2,line=2.5,cex=0.7)
mtext(paste(mvec[k]," Neighbors"),3,line=0.5,cex=0.7)
}
dev.off()



save(timing,file="timing_increasing_n_120717.RData")






load("timing_increasing_n_120717.RData")
timing





